CLIENT_TAG_MAP: dict[str, str] = {
    "audio": "device_audio",
    "security_camera": "device_camera",
    "desktop_computer": "device_pc",
    "hub": "device_other",
    "network_equipment": "device_other",
    "thermostat": "device_other",
    "laptop_computer": "device_laptop",
    "unknown_computer": "device_pc",
    "plug": "device_other",
    "watch": "device_other",
    "tablet": "device_tablet",
    "smoke_detector": "device_securityalarm",
    "game_console": "device_gameconsole",
    "digital_assistant": "device_other",
    "television": "device_tv",
    "printer": "device_printer",
    "generic": "device_other",
    "media_streamer": "device_tv",
    "garage_door": "device_other",
    "door_bell": "device_other",
    "pet_device": "device_other",
    "fan": "device_other",
    "phone": "device_phone",
    "hard_drive": "device_nas",
    "light": "device_other",
}
